package util

import (
	//"fmt"
	"regexp"
	"strings"
)

type Message struct {
	User *User
	Text string

	IsStopRequest     func() bool
	IsHomeRequest     func() bool
	IsSetHomeRequest  func() bool
	IsVisitRequest    func() bool
	IsSetPorchRequest func() bool
	VisitingUser      func() (string, bool)
	Output            func() string
}

func NewMessage(t string) *Message {
	m := new(Message)
	msg_user := regexp.MustCompile("<[^>]+>")
	tmpUser := msg_user.FindString(t)
	tmpUser = strings.Replace(tmpUser, "<", "", -1)
	tmpUser = strings.Replace(tmpUser, ">", "", -1)
	m.User = NewUser(tmpUser)
	if tmpUser == "br0xen" {
		m.User.IsOp = true
	}
	m.Text = t
	if m.User.Name != "" {
		res := strings.Split(t, "<"+m.User.Name+"> ")
		if len(res) > 0 {
			m.Text = res[1]
		}
	}

	m.IsStopRequest = func() bool {
		return (m.User.IsOp && m.Text == "!stop\n")
	}

	m.IsHomeRequest = func() bool {
		if m.User.Name != "" {
			return (m.Text == "!home\n")
		} else {
			return false
		}
	}

	m.IsSetHomeRequest = func() bool {
		if m.User.Name != "" {
			return (m.Text == "!set home\n")
		} else {
			return false
		}
	}

	m.IsSetPorchRequest = func() bool {
		if m.User.Name != "" {
			return (m.Text == "!set porch\n")
		} else {
			return false
		}
	}

	m.IsVisitRequest = func() bool {
		if m.User.Name != "" {
			return strings.HasPrefix(m.Text, "!visit ")
		} else {
			return false
		}
	}

	m.VisitingUser = func() (string, bool) {
		if m.IsVisitRequest() {
			s := strings.Replace(m.Text, "!visit ", "", -1)
			s = strings.Replace(s, "\n", "", -1)
			return s, false
		}
		return "", true
	}

	m.Output = func() string {
		if m.User.Name != "" {
			return "<" + m.User.Name + "> " + m.Text
		} else {
			return m.Text
		}
	}

	return m
}
