package util

import (
	"strings"
)

type MessageManager struct {
	/* Process takes a string and returns whether
	 * we did anything with that string or not
	 */
	Process func(inp string) bool

	/* listeners is an array of functions that
	 * tell the manager how to listen for specific
	 * text and what to do if we receive it.
	 * Each listener returns true if the input was
	 * "consumed" (i.e. - Don't send to any more
	 * listeners)
	 */
	listeners []func(inp string) bool

	/* tempListeners is an array of functions that
	 * work the same as 'listeners', but these are
	 * just temporary and higher priority than
	 * 'listeners'
	 */
	tempListeners []func(inp string) bool
}

func NewManager() *MessageManager {
	mm := new(MessageManager)
	mm.Process = func(inp string) bool {
		// TODO: send to temp listeners
		for i, listener := range listeners {
			if listener(inp) {
				return true
			}
		}
	}
	return mm
}
